﻿Module rplsViewerProc

    Public version_text = "rpls Viewer version 1.10 [2013/12/27]"
    Public playlists() As Playlist
    Public saveFileName As String = ""
    Public withHeader As Boolean = False

    Public Structure Playlist
        ' No.
        Public number As String

        ' 録画日時
        Public record_date As String

        ' 録画時間
        Public duration As String

        ' Ch.
        Public channel As String

        ' 放送局名
        Public channel_name As String

        ' 名称
        Public playlist_name As String

        ' 詳細
        Public playlist_detail As String

        ' 詳細2
        Public playlist_detail2 As String

        ' メーカー
        Public maker_id As String

        ' 機種
        Public maker_model_code As String

    End Structure

    Public Function decodeString(ba As Byte(), array_length As Integer, start_pos As Integer) As String
        ' GL,GRに呼び出し（0～3がG0～G3を表す）
        Dim GL As Integer = 0
        Dim GR As Integer = 2

        ' G0～G4への指示内容
        ' JIS : JIS 互換漢字1面
        ' ASCII : 英数
        ' HIRAGANA : プロポーショナル平仮名
        ' KATAKANA : プロポーショナル片仮名
        ' ARIB : 追加記号（ARIB外字）
        Dim G As String() = {"JIS", "ASCII", "HIRAGANA", "KATAKANA"}

        Dim decodedString As String = ""
        Dim cm As CodeMapping = New CodeMapping()

        Dim isSingleShift As Boolean = False ' シングルシフトか 2012.1.29追加
        Dim prevGL As Integer = 0 ' シングルシフト前のGL 2012.1.29追加
        Dim prevGR As Integer = 2 ' シングルシフト前のGR 2012.1.29追加

        For i As Integer = 0 To array_length - 1 Step 1
            Console.Write(ba(start_pos + i).ToString("X2") & " ")
        Next
        Console.WriteLine()

        For i As Integer = 0 To array_length - 1 Step 1
            ' 初期状態
            '
            '  【呼び出し】
            '   GL G0
            '   GR G2
            '
            '  【指示】
            '   G0 JIS 互換漢字1面
            '   G1 英数
            '   G2 プロポーショナル平仮名
            '   G3 プロポーショナル片仮名
            '
            '

            ' シングルシフトの場合、呼び出し形態を元に戻す
            If isSingleShift Then
                GL = prevGL
                GR = prevGR
                isSingleShift = False
            End If

            ' 呼び出し '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

            ' LS0 GL領域へのG0の呼び出し（ロッキングシフト）
            If ba(start_pos + i) = &HF Then
                GL = 0
                Continue For
            End If

            ' LS1 GL領域へのG1の呼び出し（ロッキングシフト）
            If ba(start_pos + i) = &HE Then
                GL = 1
                Continue For
            End If

            ' LS2 GL領域へのG2の呼び出し（ロッキングシフト） 2012.1.29 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H6E Then
                GL = 2
                i += 1
                Continue For
            End If

            ' LS3 GL領域へのG3の呼び出し（ロッキングシフト） 2012.1.29 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H6F Then
                GL = 3
                i += 1
                Continue For
            End If

            ' LS1R GRにG1を呼び出し（ロッキングシフト）（初期状態では英数）
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H7E Then
                GR = 1
                i += 1
                Continue For
            End If

            ' LS2R GRにG2を呼び出し（ロッキングシフト）（初期状態ではひらがな）
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H7D Then
                GR = 2
                i += 1
                Continue For
            End If

            ' LS3R GRにG3を呼び出し（ロッキングシフト）（初期状態ではカタカナ）
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H7C Then
                GR = 3
                i += 1
                Continue For
            End If

            ' SS2 GLにG2を呼び出し（シングルシフト） 2012.1.29 追加
            If ba(start_pos + i) = &H19 Then
                prevGL = GL
                prevGR = GR
                GL = 2
                isSingleShift = True
                i += 1 ' 2013.12.22 追加
                'Continue For ' 2013.12.22 削除
            End If

            ' SS3 GLにG3を呼び出し（シングルシフト） 2012.1.29 追加
            If ba(start_pos + i) = &H1D Then
                prevGL = GL
                prevGR = GR
                GL = 3
                isSingleShift = True
                i += 1 ' 2013.12.22 追加
                'Continue For ' 2013.12.22 削除
            End If

            ' 指示 '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

            ' GZD4 G0にASCII(JIS X0201-1976 ローマ字)を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H28 And (ba(start_pos + i + 2) = &H4A Or ba(start_pos + i + 2) = &H36) Then
                G(0) = "ASCII"
                i += 2
                Continue For
            End If

            ' G1D4 G1にASCII(JIS X0201-1976 ローマ字)を指示【初期状態】2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H29 And (ba(start_pos + i + 2) = &H4A Or ba(start_pos + i + 2) = &H36) Then
                G(1) = "ASCII"
                i += 2
                Continue For
            End If

            ' G2D4 G2にASCII(JIS X0201-1976 ローマ字)を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2A And (ba(start_pos + i + 2) = &H4A Or ba(start_pos + i + 2) = &H36) Then
                G(2) = "ASCII"
                i += 2
                Continue For
            End If

            ' G3D4 G3にASCII(JIS X0201-1976 ローマ字)を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2B And (ba(start_pos + i + 2) = &H4A Or ba(start_pos + i + 2) = &H36) Then
                G(3) = "ASCII"
                i += 2
                Continue For
            End If

            ' GZD4 G0に平仮名を指示 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H28 And (ba(start_pos + i + 2) = &H30 Or ba(start_pos + i + 2) = &H37) Then
                G(0) = "HIRAGANA"
                i += 2
                Continue For
            End If

            ' G1D4 G1に平仮名を指示 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H29 And (ba(start_pos + i + 2) = &H30 Or ba(start_pos + i + 2) = &H37) Then
                G(1) = "HIRAGANA"
                i += 2
                Continue For
            End If

            ' G2D4 G2に平仮名 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2A And (ba(start_pos + i + 2) = &H30 Or ba(start_pos + i + 2) = &H37) Then
                G(2) = "HIRAGANA"
                i += 2
                Continue For
            End If

            ' G3D4 G3に平仮名 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2B And (ba(start_pos + i + 2) = &H30 Or ba(start_pos + i + 2) = &H37) Then
                G(3) = "HIRAGANA"
                i += 2
                Continue For
            End If

            ' GZD4 G0に片仮名を指示 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H28 And (ba(start_pos + i + 2) = &H31 Or ba(start_pos + i + 2) = &H38 Or ba(start_pos + i + 2) = &H49) Then
                G(0) = "KATAKANA"
                i += 2
                Continue For
            End If

            ' G1D4 G1に片仮名を指示 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H29 And (ba(start_pos + i + 2) = &H31 Or ba(start_pos + i + 2) = &H38 Or ba(start_pos + i + 2) = &H49) Then
                G(1) = "KATAKANA"
                i += 2
                Continue For
            End If

            ' G2D4 G2に片仮名 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2A And (ba(start_pos + i + 2) = &H31 Or ba(start_pos + i + 2) = &H38 Or ba(start_pos + i + 2) = &H49) Then
                G(2) = "KATAKANA"
                i += 2
                Continue For
            End If

            ' G3D4 G3に片仮名 2013.12.23 追加
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H2B And (ba(start_pos + i + 2) = &H31 Or ba(start_pos + i + 2) = &H38 Or ba(start_pos + i + 2) = &H49) Then
                G(3) = "KATAKANA"
                i += 2
                Continue For
            End If

            ' GZDM4 G0にJIS互換漢字1面を指示【初期状態】 2013.12.23 変更
            '                                                            ここで0x28が省略されている
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And (ba(start_pos + i + 2) = &H39 Or ba(start_pos + i + 2) = &H42) Then
                G(0) = "JIS"
                i += 2
                Continue For
            End If

            ' G1DM4 G1にJIS互換漢字1面を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H29 And (ba(start_pos + i + 2) = &H39 Or ba(start_pos + i + 2) = &H42) Then
                G(1) = "JIS"
                i += 3
                Continue For
            End If

            ' G2DM4 G2にJIS互換漢字1面を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H2A And (ba(start_pos + i + 2) = &H39 Or ba(start_pos + i + 2) = &H42) Then
                G(2) = "JIS"
                i += 3
                Continue For
            End If

            ' G3DM4 G3にJIS互換漢字1面を指示 2013.12.23 変更
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H2B And (ba(start_pos + i + 2) = &H39 Or ba(start_pos + i + 2) = &H42) Then
                G(3) = "JIS"
                i += 3
                Continue For
            End If

            ' GZDM4 G0への94n文字集合の指示
            '			                                                 ここで0x28が省略されている
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H3B Then
                G(0) = "ARIB"
                i += 2
                Continue For
            End If

            ' G1DM4 G1への94n文字集合の指示
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H29 And ba(start_pos + i + 3) = &H3B Then
                G(1) = "ARIB"
                i += 3
                Continue For
            End If

            ' G2DM4 G2への94n文字集合の指示
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H2A And ba(start_pos + i + 3) = &H3B Then
                G(2) = "ARIB"
                i += 3
                Continue For
            End If

            ' G3DM4 G3への94n文字集合の指示
            If ba(start_pos + i) = &H1B And ba(start_pos + i + 1) = &H24 And ba(start_pos + i + 2) = &H2B And ba(start_pos + i + 3) = &H3B Then
                G(3) = "ARIB"
                i += 3
                Continue For
            End If

            ' CR,LFをデータに忠実に処理するように変更
            If ba(start_pos + i) = &HA Then
                decodedString = decodedString + Chr(10)
                Continue For
            End If

            ' CR,LFをデータに忠実に処理するように変更
            If ba(start_pos + i) = &HD Then
                If ba(start_pos + i + 1) = &HA Then
                    decodedString = decodedString + Chr(13)
                Else
                    decodedString = decodedString + Chr(13) + Chr(10)
                End If
                Continue For
            End If

            If ba(start_pos + i) = &H20 Then
                decodedString = decodedString + " "
                Continue For
            End If

            ' MSZ 中型サイズ
            If ba(start_pos + i) = &H89 Then
                ' 何もしない
                Continue For
            End If

            ' NSZ 標準サイズ
            If ba(start_pos + i) = &H8A Then
                ' 何もしない
                Continue For
            End If

            '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

            ' GL
            If ba(start_pos + i) < &H80 Then
                If G(GL) = "HIRAGANA" Or G(GL) = "KATAKANA" Then
                    ' ひらがな・カタカナ共通
                    '
                    ' ARIB-STD B24では
                    ' 77 78 79 7A 7B 7C 7D 7E は、
                    ' ゝ ゞ ー 。 「 」 、 ・ に対応する。
                    '
                    If ba(start_pos + i) = &H77 Then
                        decodedString = decodedString + "ゝ"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H78 Then
                        decodedString = decodedString + "ゞ"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H79 Then
                        decodedString = decodedString + "ー"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H7A Then
                        decodedString = decodedString + "。"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H7B Then
                        decodedString = decodedString + "「"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H7C Then
                        decodedString = decodedString + "」"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H7D Then
                        decodedString = decodedString + "、"
                        Continue For
                    End If

                    If ba(start_pos + i) = &H7E Then
                        decodedString = decodedString + "・"
                        Continue For
                    End If
                End If

                If G(GL) = "ASCII" Then
                    ' ASCII
                    decodedString = decodedString + Chr(ba(start_pos + i))
                    Continue For
                End If

                ' 2013.12.22 追加
                If G(GL) = "HIRAGANA" Then
                    ' EUCひらがな
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(&HA400 + ba(start_pos + i) - &H8000))
                    Continue For
                End If

                ' 2013.12.22 追加
                If G(GL) = "KATAKANA" Then
                    ' EUCカタカナ
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(&HA500 + ba(start_pos + i) - &H8000))
                    Continue For
                End If

                If G(GL) = "JIS" Then
                    ' JIS
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(ba(start_pos + i) * 256 + ba(start_pos + i + 1)))
                    i += 1
                    Continue For
                End If

                If G(GL) = "ARIB" Then
                    ' 追加記号（ARIB外字）
                    decodedString = decodedString + cm.ARIBgaiji(ba(start_pos + i) * 256 + ba(start_pos + i + 1))
                    i += 1
                    Continue For
                End If
            End If

            ' GR
            If ba(start_pos + i) >= &H80 Then
                If G(GR) = "HIRAGANA" Or G(GR) = "KATAKANA" Then
                    ' ひらがな・カタカナ共通
                    '
                    ' ARIB-STD B24では
                    ' F7 F8 F9 FA FB FC FD FE は、
                    ' ゝ ゞ ー 。 「 」 、 ・ に対応する。
                    '
                    ' 2014.6.28 77～7EをF7～FEに修正
                    If ba(start_pos + i) = &HF7 Then
                        decodedString = decodedString + "ゝ"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HF8 Then
                        decodedString = decodedString + "ゞ"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HF9 Then
                        decodedString = decodedString + "ー"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HFA Then
                        decodedString = decodedString + "。"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HFB Then
                        decodedString = decodedString + "「"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HFC Then
                        decodedString = decodedString + "」"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HFD Then
                        decodedString = decodedString + "、"
                        Continue For
                    End If

                    If ba(start_pos + i) = &HFE Then
                        decodedString = decodedString + "・"
                        Continue For
                    End If
                End If

                If G(GR) = "ASCII" Then
                    ' EUC ASCII
                    decodedString = decodedString + Chr(ba(start_pos + i) - &H80)
                    Continue For
                End If

                If G(GR) = "HIRAGANA" Then
                    ' EUCひらがな
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(&HA400 + ba(start_pos + i) - &H8080))
                    Continue For
                End If

                If G(GR) = "KATAKANA" Then
                    ' EUCカタカナ
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(&HA500 + ba(start_pos + i) - &H8080))
                    Continue For
                End If

                ' 2013.12.22 追加
                If G(GR) = "JIS" Then
                    ' JIS
                    decodedString = decodedString + ChrW(cm.JIStoUnicode(ba(start_pos + i) * 256 + ba(start_pos + i + 1) - &H8080))
                    i += 1
                    Continue For
                End If

                If G(GR) = "ARIB" Then
                    ' 追加記号（ARIB外字）
                    decodedString = decodedString + cm.ARIBgaiji(ba(start_pos + i) * 256 + ba(start_pos + i + 1) - &H8080)
                    i += 1
                    Continue For
                End If
            End If

        Next i
        decodeString = decodedString
    End Function

    Public Function GetFiles(path As String, searchPattern As String) As String()
        GetFiles = System.IO.Directory.GetFiles(path, searchPattern, System.IO.SearchOption.TopDirectoryOnly)
    End Function

End Module
